@@ -1,5 +1,5 @@
 from superpowered.main import init
-from superpowered.knowledge_base import KnowledgeBase, KnowledgeBaseDocument, create_knowledge_base, get_knowledge_base, list_knowledge_bases, list_documents_in_kb, add_file_to_kb, add_directory_to_kb, query, add_document_to_kb, add_url_to_kb
+from superpowered.knowledge_base import KnowledgeBase, KnowledgeBaseDocument, create_knowledge_base, get_knowledge_base, list_knowledge_bases, list_documents_in_kb, add_file_to_kb, add_directory_to_kb, query, real_time_query, add_document_to_kb, add_url_to_kb
 
 __all__ = [
     "init",
@@ -12,6 +12,7 @@ __all__ = [
     "list_knowledge_bases",
     "list_documents_in_kb",
     "query",
+    "real_time_query",
     "add_document_to_kb",
     "add_url_to_kb",
-]
\ No newline at end of file
+]
