@@ -1,4 +1,7 @@
 from superpowered.main import BASE_URL, _format_http_response, get_headers
+
+import base64
+import hashlib
 import requests
 import os
 import PyPDF2
@@ -124,6 +127,7 @@ def get_knowledge_bases():
         knowledge_bases[knowledge_base['id']] = kb
     return knowledge_bases
 
+
 def create_knowledge_base(title: str, supp_id: str = None, description: str = None):
     """
     create_knowledge_base() is a convenience function that creates a KnowledgeBase object and then calls its create() method
@@ -132,6 +136,7 @@ def create_knowledge_base(title: str, supp_id: str = None, description: str = No
     resp = kb.create()
     return kb
 
+
 def get_knowledge_base(title: str):
     """
     get_knowledge_base() is a convenience function that returns a KnowledgeBase object for an existing knowledge base, given its title
@@ -152,6 +157,7 @@ def get_knowledge_base(title: str):
     else:
         raise Exception('Knowledge base title not found: ' + title)
 
+
 def list_knowledge_bases(verbose=True):
     """
     list_knowledge_bases() is a convenience function that returns a dictionary of all KnowledgeBase objects for an account
@@ -163,6 +169,7 @@ def list_knowledge_bases(verbose=True):
             print (f"id: {kb_obj.kb_id}\ntitle: {kb_obj.title}\n")
     return knowledge_bases
 
+
 def list_documents_in_kb(kb_title: str, verbose=True):
     """
     list_documents_in_kb() is a convenience function that returns a dictionary of KnowledgeBaseDocument objects for an existing knowledge base, given its title
@@ -175,55 +182,6 @@ def list_documents_in_kb(kb_title: str, verbose=True):
             print (f"title: {document_obj.title}\n\n")
     return documents
 
-def read_file(file_path: str):
-    """
-    read a file and return its contents as a string
-    """
-    # infer the file type from the file extension
-    file_type = file_path.split(".")[-1]
-
-    if file_type == "pdf":
-        # Open the PDF file
-        with open(file_path, 'rb') as file:
-            # Create a PDF reader object
-            pdf_reader = PyPDF2.PdfReader(file)
-
-            # Initialize an empty string to store the extracted text
-            extracted_text = ""
-
-            # Loop through all the pages in the PDF file
-            for page_num in range(len(pdf_reader.pages)):
-                # Extract the text from the current page
-                page_text = pdf_reader.pages[page_num].extract_text()
-
-                # Add the extracted text to the final text
-                extracted_text += page_text 
-        return extracted_text
-    elif file_type == "txt":
-        with open(file_path, 'r') as f:
-            # read the file and convert it to a string
-            content = f.read()
-        return content
-    elif file_type == "docx":
-        # Create a docx Document object
-        doc = docx.Document(file_path)
-
-        # Initialize an empty string to store the extracted text
-        extracted_text = ""
-
-        # Loop through all the paragraphs in the docx file
-        for paragraph in doc.paragraphs:
-            # Add the extracted text to the final text
-            extracted_text += paragraph.text + "\n"
-
-        return extracted_text
-    elif file_type == "md":
-        with open(file_path, 'r') as f:
-            # read the file and convert it to a string
-            content = f.read()
-        return content
-    else:
-        raise Exception("File type not supported. Supported file types are: .txt, .pdf, .docx, and .md")
     
 def add_document_to_kb(kb_title: str, content: str, title: str = None, link_to_source: str = None, supp_id: str = None, description: str = None):
     """
@@ -233,18 +191,42 @@ def add_document_to_kb(kb_title: str, content: str, title: str = None, link_to_s
     kb = get_knowledge_base(kb_title)
     kb.add_document(content=content, title=title, link_to_source=link_to_source, supp_id=supp_id, description=description)
 
-def add_file_to_kb(kb_title: str, file_path: str, title: str = None, link_to_source: str = None, supp_id: str = None, description: str = None):
+
+def add_file_to_kb(kb_title: str, file_path: str, supp_id: str = None, description: str = None):
     """
     add a Document to a knowledge base by reading the contents of a file - currently supports .txt, .pdf, .docx, and .md files
     """
-    content = read_file(file_path)
-    
-    # make the title the file name if it's not provided
-    if title is None:
-        title = file_path.split("/")[-1]
-    
+    # get knowledge_base_id
     kb = get_knowledge_base(kb_title)
-    kb.add_document(content=content, title=title, link_to_source=link_to_source, supp_id=supp_id, description=description)
+    knowledge_base_id = kb.kb_id
+
+    # set url
+    url = f'{BASE_URL}knowledge_bases/{knowledge_base_id}/documents/request_signed_file_url'
+
+    # read contents of file
+    with open(file_path, 'rb') as f:
+        contents = f.read()
+        md5 = hashlib.md5(contents)
+        encoded_md5 = base64.b64encode(md5.digest()).decode('utf-8')
+
+    request_upload_payload = {
+        'filename': file_path.split("/")[-1],
+        'method': 'PUT',
+        'encoded_md5': encoded_md5,
+    }
+    # get the presigned url
+    resp = _format_http_response(requests.post(url, headers=get_headers(), json=request_upload_payload))
+    upload_url = resp['body']['temporary_url']
+
+    # upload the file
+    headers = {
+        'Content-MD5': encoded_md5,
+    }
+    resp = requests.put(upload_url, data=contents, headers=headers)
+    if resp.status_code != 200:
+        raise Exception(f'Error uploading file: {resp.content}')
+    return resp.content
+
 
 def add_directory_to_kb(kb_title: str, directory_path: str, verbose=False):
     """
@@ -262,6 +244,7 @@ def add_directory_to_kb(kb_title: str, directory_path: str, verbose=False):
         if verbose: print("Adding:", file)
         add_file_to_kb(kb_title, file_path=file)
 
+
 def add_url_to_kb(kb_title: str, url: str, title: str = None, link_to_source: str = None, supp_id: str = None, description: str = None):
     """
     add a Document to a knowledge base by scraping the contents of a URL
@@ -279,6 +262,7 @@ def add_url_to_kb(kb_title: str, url: str, title: str = None, link_to_source: st
     kb = get_knowledge_base(kb_title)
     kb.add_document(content=content, title=title, link_to_source=link_to_source, supp_id=supp_id, description=description)
 
+
 def query(query: str, kb_titles: list = [], kb_ids: list = [], retriever_top_k: int = 100, reranker_top_k: int = 5, extract_and_summarize: bool = False):
     """
     Can take either a list of kb_titles or a list of kb_ids (or a mix of both)
@@ -310,4 +294,27 @@ def query(query: str, kb_titles: list = [], kb_ids: list = [], retriever_top_k:
         'summarize_results': extract_and_summarize,
     }
     resp = _format_http_response(requests.post(url, headers=get_headers(), json=payload))
+    return resp["body"]
+
+
+def real_time_query(query: str, passages: list[str], top_k: int = 10, max_chunk_length: int = 500, summarize_results: bool = False):
+    """
+    Takes a query and a list of passages to run the query against. Returns the top k passages that are most relevant to the query.
+    - NOTE that this does not use knowledge bases. All documents to be searched must be provided in the passages list, and these passages
+      will not be saved to any knowledge base.
+    """
+    # make sure passages is not empty
+    if len(passages) == 0:
+        raise Exception("Must provide at least one passage")
+
+    # make the request
+    url = BASE_URL + 'realtime_query'
+    payload = {
+        'query': query,
+        'passages': passages,
+        'top_k': top_k,
+        'max_chunk_length': max_chunk_length,
+        'summarize_results': summarize_results,
+    }
+    resp = _format_http_response(requests.post(url, headers=get_headers(), json=payload))
     return resp["body"]
\ No newline at end of file
